package ch.nuage.arduage;

import android.app.Activity;

import com.ftdi.j2xx.D2xxManager;
import com.ftdi.j2xx.D2xxManager.D2xxException;
import com.ftdi.j2xx.FT_Device;

public abstract class Ardunio {

	private static final int BAUD = 9600;
	private final Activity activity;

	int b = 0;

	private final D2xxManager ftD2xx;

	int g = 0;

	private boolean getTemp = false;

	private boolean help = false;

	private boolean isRunning = false;

	private boolean isStopRequested = false;

	int r = 0;

	private boolean setLed = false;

	public Ardunio(Activity activity) throws D2xxException {
		ftD2xx = D2xxManager.getInstance(activity);
		this.activity = activity;
	}

	public void help() {
		help = true;
	}

	public abstract void info(String txt);

	/**
	 * This function is running in the background (until shutdown is called)
	 */
	private void runInBg() {
		while (!isStopRequested) {
			try {

				//
				int cntDev = ftD2xx.createDeviceInfoList(activity);
				if (cntDev >= 1) {
					info("Device found: " + cntDev);
					FT_Device dev = ftD2xx.openByIndex(activity, 0);
					if (dev == null) {
						info("Device error.");
						if (isStopRequested) {
							return;
						}
						Thread.sleep(1000);
					} else {
						dev.setBitMode((byte) 0, D2xxManager.FT_BITMODE_RESET);
						dev.setBaudRate(BAUD);
						dev.setDataCharacteristics(D2xxManager.FT_DATA_BITS_8,
								D2xxManager.FT_STOP_BITS_1,
								D2xxManager.FT_PARITY_NONE);
						dev.setFlowControl(D2xxManager.FT_FLOW_NONE,
								(byte) 0x0b, (byte) 0x0d);

						info("Device configured");
						byte[] b = new byte[1];
						StringBuffer buff = new StringBuffer();
						while (!isStopRequested) {

							try {
								int c = 1;
								while (c > 0) {
									c = dev.read(b, 1, 200);
									if (c > 0) {
										buff.append((char) b[0]);
									}
								}
							} catch (Exception e) {
								// e.printStackTrace();
								//
							}

							info(buff.toString());

							if (setLed) {
								String s = "r" + this.r + "g" + this.g + "b"
										+ this.b;
								dev.write(s.getBytes());
								setLed = false;
							}

							if (getTemp) {
								buff = new StringBuffer();
								dev.write("t".getBytes());
								getTemp = false;
							}

							if (help) {
								dev.write("h".getBytes());
								buff = new StringBuffer();
								help = false;
							}

							Thread.sleep(100);
						}

						Thread.sleep(1000);

						dev.close();
					}
				} else {
					info("No device found");
				}

				Thread.sleep(100);
			} catch (Throwable t) {
				info("Error: " + t.getMessage());
				t.printStackTrace();
			}

		}
		isRunning = false;
	}

	public void setLedB(int progress) {
		setLed = true;
		b = progress;
	}

	public void setLedG(int progress) {
		setLed = true;
		g = progress;
	}

	public void setLedR(int progress) {
		setLed = true;
		r = progress;
	}

	/**
	 * Call this function to shutdown this thread.
	 */
	public void shutdown() {
		isStopRequested = true;
		while (isRunning) {
			Thread.yield();
		}
	}

	/**
	 * Call this function to start this thread.
	 */
	public void start() {

		if (isRunning) {
			throw new RuntimeException("Background task is already running!");
		}

		isRunning = true;
		isStopRequested = false;

		Runnable r = new Runnable() {
			@Override
			public void run() {
				runInBg();
			}
		};
		Thread t = new Thread(r);
		t.start();

	}

	public void temperature() {
		getTemp = true;
	}

}
